@@ -259,7 +259,7 @@ A:link, A:visited, A:active { text-decoration: underline }
 			    } else {
 				$id = strtoupper($name);
 				$m = strtolower($module);
-				$url = "html/libxml-$m.html#$id";
+				$url = "html/libxml-$module.html#$id";
 				$results[$name] = array($relevance,$type,
 						$module, $desc, $name, $url);
 			    }
@@ -284,7 +284,7 @@ A:link, A:visited, A:active { text-decoration: underline }
 			    } else {
 				$id = strtoupper($name);
 				$m = strtolower($module);
-				$url = "XSLT/html/libxslt-$m.html#$id";
+				$url = "XSLT/html/libxslt-$module.html#$id";
 				$results[$name] = array($relevance,$type,
 						$module, $desc, $name, $url);
 			    }
@@ -386,9 +386,6 @@ A:link, A:visited, A:active { text-decoration: underline }
 			mysql_free_result($result);
 		    }
 		}
-		if (($j <= 0) && ($k <= 0)) {
-		    echo "<p> No result found for $word\n";
-		}
 	    }
 	    mysql_close($link);
 	    $nb = count($results);
