@@ -52,6 +52,12 @@
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
 #ifdef HAVE_STRINGS_H
 #include <strings.h>
 #endif
@@ -62,7 +68,6 @@
 #include <libxml/uri.h>
 #include <libxml/nanoftp.h>
 #include <libxml/globals.h>
-#include <config.h>
 
 /* #define DEBUG_FTP 1  */
 #ifdef STANDALONE
@@ -71,17 +76,36 @@
 #endif
 #endif
 
+
+#ifdef __MINGW32__
+#define _WINSOCKAPI_
+#include <wsockcompat.h>
+#include <winsock2.h>
+#undef SOCKLEN_T
+#define SOCKLEN_T unsigned int
+#endif
+
+
 /**
  * A couple portability macros
  */
 #ifndef _WINSOCKAPI_
+#ifndef __BEOS__
 #define closesocket(s) close(s)
+#endif
 #define SOCKET int
 #endif
 #if defined(VMS) || defined(__VMS)
 #define SOCKLEN_T unsigned int
 #endif
 
+#ifdef __BEOS__
+#ifndef PF_INET
+#define PF_INET AF_INET
+#endif
+#endif
+
+
 #define FTP_COMMAND_OK		200
 #define FTP_SYNTAX_ERROR	500
 #define FTP_GET_PASSWD		331
@@ -119,7 +143,8 @@ static char *proxyPasswd = NULL;/* passwd for proxy authentication */
 static int proxyType = 0;	/* uses TYPE or a@b ? */
 
 #ifdef SUPPORT_IP6
-static int have_ipv6() {
+static
+int have_ipv6(void) {
     int s;
 
     s = socket (AF_INET6, SOCK_STREAM, 0);
@@ -131,6 +156,18 @@ static int have_ipv6() {
 }
 #endif
 
+/**
+ * xmlFTPErrMemory:
+ * @extra:  extra informations
+ *
+ * Handle an out of memory condition
+ */
+static void
+xmlFTPErrMemory(const char *extra)
+{
+    __xmlSimpleError(XML_FROM_FTP, XML_ERR_NO_MEMORY, NULL, NULL, extra);
+}
+
 /**
  * xmlNanoFTPInit:
  *
@@ -633,7 +670,10 @@ xmlNanoFTPNewCtxt(const char *URL) {
     char *unescaped;
 
     ret = (xmlNanoFTPCtxtPtr) xmlMalloc(sizeof(xmlNanoFTPCtxt));
-    if (ret == NULL) return(NULL);
+    if (ret == NULL) {
+        xmlFTPErrMemory("allocating FTP context");
+        return(NULL);
+    }
 
     memset(ret, 0, sizeof(xmlNanoFTPCtxt));
     ret->port = 21;
@@ -773,6 +813,7 @@ xmlNanoFTPGetMore(void *ctx) {
      */
     if ((len = recv(ctxt->controlFd, &ctxt->controlBuf[ctxt->controlBufIndex],
 		    size, 0)) < 0) {
+	__xmlIOErr(XML_FROM_FTP, 0, "recv failed");
 	closesocket(ctxt->controlFd); ctxt->controlFd = -1;
         ctxt->controlFd = -1;
         return(-1);
@@ -897,9 +938,7 @@ xmlNanoFTPCheckResponse(void *ctx) {
 	case 0:
 	    return(0);
 	case -1:
-#ifdef DEBUG_FTP
-	    perror("select");
-#endif
+	    __xmlIOErr(XML_FROM_FTP, 0, "select");
 	    return(-1);
 			
     }
@@ -928,7 +967,10 @@ xmlNanoFTPSendUser(void *ctx) {
     xmlGenericError(xmlGenericErrorContext, "%s", buf);
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
-    if (res < 0) return(res);
+    if (res < 0) {
+	__xmlIOErr(XML_FROM_FTP, 0, "send failed");
+	return(res);
+    }
     return(0);
 }
 
@@ -953,7 +995,10 @@ xmlNanoFTPSendPasswd(void *ctx) {
     xmlGenericError(xmlGenericErrorContext, "%s", buf);
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
-    if (res < 0) return(res);
+    if (res < 0) {
+	__xmlIOErr(XML_FROM_FTP, 0, "send failed");
+	return(res);
+    }
     return(0);
 }
 
@@ -971,8 +1016,7 @@ int
 xmlNanoFTPQuit(void *ctx) {
     xmlNanoFTPCtxtPtr ctxt = (xmlNanoFTPCtxtPtr) ctx;
     char buf[200];
-    int len;
-    int res;
+    int len, res;
 
     snprintf(buf, sizeof(buf), "QUIT\r\n");
     len = strlen(buf);
@@ -980,6 +1024,10 @@ xmlNanoFTPQuit(void *ctx) {
     xmlGenericError(xmlGenericErrorContext, "%s", buf); /* Just to be consistent, even though we know it can't have a % in it */
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
+    if (res < 0) {
+	__xmlIOErr(XML_FROM_FTP, 0, "send failed");
+	return(res);
+    }
     return(0);
 }
 
@@ -998,7 +1046,7 @@ xmlNanoFTPConnect(void *ctx) {
     struct hostent *hp;
     int port;
     int res;
-    int addrlen;
+    int addrlen = sizeof (struct sockaddr_in);
 
     if (ctxt == NULL)
 	return(-1);
@@ -1020,36 +1068,45 @@ xmlNanoFTPConnect(void *ctx) {
 
 #ifdef SUPPORT_IP6
     if (have_ipv6 ()) {
-	struct addrinfo hints, *res, *result;
+	struct addrinfo hints, *tmp, *result;
 
 	result = NULL;
 	memset (&hints, 0, sizeof(hints));
 	hints.ai_socktype = SOCK_STREAM;
 
 	if (proxy) {
-	    if (getaddrinfo (proxy, NULL, &hints, &result) != 0)
+	    if (getaddrinfo (proxy, NULL, &hints, &result) != 0) {
+		__xmlIOErr(XML_FROM_FTP, 0, "getaddrinfo failed");
 		return (-1);
+	    }
 	}
 	else
-	    if (getaddrinfo (ctxt->hostname, NULL, &hints, &result) != 0)
+	    if (getaddrinfo (ctxt->hostname, NULL, &hints, &result) != 0) {
+		__xmlIOErr(XML_FROM_FTP, 0, "getaddrinfo failed");
 		return (-1);
+	    }
 
-	for (res = result; res; res = res->ai_next)
-	    if (res->ai_family == AF_INET || res->ai_family == AF_INET6)
+	for (tmp = result; tmp; tmp = tmp->ai_next)
+	    if (tmp->ai_family == AF_INET || tmp->ai_family == AF_INET6)
 		break;
 
-	if (res) {
-	    if (res->ai_family == AF_INET6) {
-		memcpy (&ctxt->ftpAddr, res->ai_addr, res->ai_addrlen);
+	if (!tmp) {
+	    if (result)
+		freeaddrinfo (result);
+	    return (-1);
+	}
+	else {
+	    if (tmp->ai_family == AF_INET6) {
+		memcpy (&ctxt->ftpAddr, tmp->ai_addr, tmp->ai_addrlen);
 		((struct sockaddr_in6 *) &ctxt->ftpAddr)->sin6_port = htons (port);
 		ctxt->controlFd = socket (AF_INET6, SOCK_STREAM, 0);
 	    }
 	    else {
-		memcpy (&ctxt->ftpAddr, res->ai_addr, res->ai_addrlen);
+		memcpy (&ctxt->ftpAddr, tmp->ai_addr, tmp->ai_addrlen);
 		((struct sockaddr_in *) &ctxt->ftpAddr)->sin_port = htons (port);
 		ctxt->controlFd = socket (AF_INET, SOCK_STREAM, 0);
 	    }
-	    addrlen = res->ai_addrlen;
+	    addrlen = tmp->ai_addrlen;
 	    freeaddrinfo (result);
 	}
     }
@@ -1060,8 +1117,10 @@ xmlNanoFTPConnect(void *ctx) {
 	    hp = gethostbyname (proxy);
 	else
 	    hp = gethostbyname (ctxt->hostname);
-	if (hp == NULL)
+	if (hp == NULL) {
+	    __xmlIOErr(XML_FROM_FTP, 0, "gethostbyname failed");
 	    return (-1);
+	}
 
     /*
      * Prepare the socket
@@ -1074,14 +1133,17 @@ xmlNanoFTPConnect(void *ctx) {
 	addrlen = sizeof (struct sockaddr_in);
     }
 
-    if (ctxt->controlFd < 0)
+    if (ctxt->controlFd < 0) {
+	__xmlIOErr(XML_FROM_FTP, 0, "socket failed");
         return(-1);
+    }
 
     /*
      * Do the connect.
      */
     if (connect(ctxt->controlFd, (struct sockaddr *) &ctxt->ftpAddr,
 	    addrlen) < 0) {
+	__xmlIOErr(XML_FROM_FTP, 0, "Failed to create a connection");
         closesocket(ctxt->controlFd); ctxt->controlFd = -1;
         ctxt->controlFd = -1;
 	return(-1);
@@ -1148,6 +1210,7 @@ xmlNanoFTPConnect(void *ctx) {
 #endif
 	    res = send(ctxt->controlFd, buf, len, 0);
 	    if (res < 0) {
+		__xmlIOErr(XML_FROM_FTP, 0, "send failed");
 		closesocket(ctxt->controlFd);
 		ctxt->controlFd = -1;
 	        return(res);
@@ -1169,6 +1232,7 @@ xmlNanoFTPConnect(void *ctx) {
 #endif
 		    res = send(ctxt->controlFd, buf, len, 0);
 		    if (res < 0) {
+			__xmlIOErr(XML_FROM_FTP, 0, "send failed");
 			closesocket(ctxt->controlFd);
 			ctxt->controlFd = -1;
 			return(res);
@@ -1209,6 +1273,7 @@ xmlNanoFTPConnect(void *ctx) {
 #endif
 		res = send(ctxt->controlFd, buf, len, 0);
 		if (res < 0) {
+		    __xmlIOErr(XML_FROM_FTP, 0, "send failed");
 		    closesocket(ctxt->controlFd); ctxt->controlFd = -1;
 		    ctxt->controlFd = -1;
 		    return(res);
@@ -1239,6 +1304,7 @@ xmlNanoFTPConnect(void *ctx) {
 #endif
 		res = send(ctxt->controlFd, buf, len, 0);
 		if (res < 0) {
+		    __xmlIOErr(XML_FROM_FTP, 0, "send failed");
 		    closesocket(ctxt->controlFd); ctxt->controlFd = -1;
 		    ctxt->controlFd = -1;
 		    return(res);
@@ -1260,6 +1326,7 @@ xmlNanoFTPConnect(void *ctx) {
 #endif
 		res = send(ctxt->controlFd, buf, len, 0);
 		if (res < 0) {
+		    __xmlIOErr(XML_FROM_FTP, 0, "send failed");
 		    closesocket(ctxt->controlFd); ctxt->controlFd = -1;
 		    ctxt->controlFd = -1;
 		    return(res);
@@ -1321,8 +1388,8 @@ xmlNanoFTPConnect(void *ctx) {
 	case 2:
 	    break;
 	case 3:
-	    xmlGenericError(xmlGenericErrorContext,
-		    "FTP server asking for ACCNT on anonymous\n");
+	    __xmlIOErr(XML_FROM_FTP, XML_FTP_ACCNT,
+		       "FTP server asking for ACCNT on anonymous\n");
 	case 1:
 	case 4:
 	case 5:
@@ -1397,7 +1464,10 @@ xmlNanoFTPCwd(void *ctx, char *directory) {
     xmlGenericError(xmlGenericErrorContext, "%s", buf);
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
-    if (res < 0) return(res);
+    if (res < 0) {
+	__xmlIOErr(XML_FROM_FTP, 0, "send failed");
+	return(res);
+    }
     res = xmlNanoFTPGetResponse(ctxt);
     if (res == 4) {
 	return(-1);
@@ -1442,7 +1512,10 @@ xmlNanoFTPDele(void *ctx, char *file) {
     xmlGenericError(xmlGenericErrorContext, "%s", buf);
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
-    if (res < 0) return(res);
+    if (res < 0) {
+	__xmlIOErr(XML_FROM_FTP, 0, "send failed");
+	return(res);
+    }
     res = xmlNanoFTPGetResponse(ctxt);
     if (res == 4) {
 	return(-1);
@@ -1493,8 +1566,7 @@ xmlNanoFTPGetConnection(void *ctx) {
     }
 
     if (ctxt->dataFd < 0) {
-	xmlGenericError (xmlGenericErrorContext,
-		"xmlNanoFTPGetConnection: failed to create socket\n");
+	__xmlIOErr(XML_FROM_FTP, 0, "socket failed");
 	return (-1);
     }
 
@@ -1511,6 +1583,7 @@ xmlNanoFTPGetConnection(void *ctx) {
 #endif
 	res = send(ctxt->controlFd, buf, len, 0);
 	if (res < 0) {
+	    __xmlIOErr(XML_FROM_FTP, 0, "send failed");
 	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return(res);
 	}
@@ -1532,7 +1605,7 @@ xmlNanoFTPGetConnection(void *ctx) {
 #ifdef SUPPORT_IP6
 	if ((ctxt->ftpAddr).ss_family == AF_INET6) {
 	    if (sscanf (cur, "%u", &temp[0]) != 1) {
-		xmlGenericError (xmlGenericErrorContext,
+		__xmlIOErr(XML_FROM_FTP, XML_FTP_EPSV_ANSWER,
 			"Invalid answer to EPSV\n");
 		if (ctxt->dataFd != -1) {
 		    closesocket (ctxt->dataFd); ctxt->dataFd = -1;
@@ -1547,7 +1620,7 @@ xmlNanoFTPGetConnection(void *ctx) {
 	{
 	    if (sscanf (cur, "%u,%u,%u,%u,%u,%u", &temp[0], &temp[1], &temp[2],
 		&temp[3], &temp[4], &temp[5]) != 6) {
-		xmlGenericError (xmlGenericErrorContext,
+		__xmlIOErr(XML_FROM_FTP, XML_FTP_PASV_ANSWER,
 			"Invalid answer to PASV\n");
 		if (ctxt->dataFd != -1) {
 		    closesocket (ctxt->dataFd); ctxt->dataFd = -1;
@@ -1560,8 +1633,7 @@ xmlNanoFTPGetConnection(void *ctx) {
 	}
 
 	if (connect(ctxt->dataFd, (struct sockaddr *) &dataAddr, dataAddrLen) < 0) {
-	    xmlGenericError(xmlGenericErrorContext,
-		    "Failed to create a data connection\n");
+	    __xmlIOErr(XML_FROM_FTP, 0, "Failed to create a data connection");
 	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return (-1);
 	}
@@ -1575,24 +1647,14 @@ xmlNanoFTPGetConnection(void *ctx) {
 	    ((struct sockaddr_in *)&dataAddr)->sin_port = 0;
 
 	if (bind(ctxt->dataFd, (struct sockaddr *) &dataAddr, dataAddrLen) < 0) {
-	    xmlGenericError(xmlGenericErrorContext,
-		    "Failed to bind a port\n");
+	    __xmlIOErr(XML_FROM_FTP, 0, "bind failed");
 	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return (-1);
 	}
         getsockname(ctxt->dataFd, (struct sockaddr *) &dataAddr, &dataAddrLen);
 
 	if (listen(ctxt->dataFd, 1) < 0) {
-#ifdef SUPPORT_IP6
-	    if ((ctxt->ftpAddr).ss_family == AF_INET6)
-		xmlGenericError (xmlGenericErrorContext,
-			"Could not listen on port %d\n",
-			ntohs (((struct sockaddr_in6 *)&dataAddr)->sin6_port));
-	    else
-#endif
-		xmlGenericError (xmlGenericErrorContext,
-			"Could not listen on port %d\n",
-			ntohs (((struct sockaddr_in *)&dataAddr)->sin_port));
+	    __xmlIOErr(XML_FROM_FTP, 0, "listen failed");
 	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return (-1);
 	}
@@ -1601,7 +1663,7 @@ xmlNanoFTPGetConnection(void *ctx) {
 	    char buf6[INET6_ADDRSTRLEN];
 	    inet_ntop (AF_INET6, &((struct sockaddr_in6 *)&dataAddr)->sin6_addr,
 		    buf6, INET6_ADDRSTRLEN);
-	    adp = buf6;
+	    adp = (unsigned char *) buf6;
 	    portp = (unsigned char *) &((struct sockaddr_in6 *)&dataAddr)->sin6_port;
 	    snprintf (buf, sizeof(buf), "EPRT |2|%s|%s|\r\n", adp, portp);
         } else
@@ -1622,6 +1684,7 @@ xmlNanoFTPGetConnection(void *ctx) {
 
 	res = send(ctxt->controlFd, buf, len, 0);
 	if (res < 0) {
+	    __xmlIOErr(XML_FROM_FTP, 0, "send failed");
 	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return(res);
 	}
@@ -1855,6 +1918,7 @@ xmlNanoFTPList(void *ctx, ftpListCallback callback, void *userData,
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
     if (res < 0) {
+	__xmlIOErr(XML_FROM_FTP, 0, "send failed");
 	closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	return(res);
     }
@@ -1895,9 +1959,7 @@ xmlNanoFTPList(void *ctx, ftpListCallback callback, void *userData,
 	}
 
 	if ((len = recv(ctxt->dataFd, &buf[indx], sizeof(buf) - (indx + 1), 0)) < 0) {
-#ifdef DEBUG_FTP
-	    perror("recv");
-#endif
+	    __xmlIOErr(XML_FROM_FTP, 0, "recv");
 	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    ctxt->dataFd = -1;
 	    return(-1);
@@ -1949,6 +2011,7 @@ xmlNanoFTPGetSocket(void *ctx, const char *filename) {
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
     if (res < 0) {
+	__xmlIOErr(XML_FROM_FTP, 0, "send failed");
 	closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	return(res);
     }
@@ -1968,6 +2031,7 @@ xmlNanoFTPGetSocket(void *ctx, const char *filename) {
 #endif
     res = send(ctxt->controlFd, buf, len, 0);
     if (res < 0) {
+	__xmlIOErr(XML_FROM_FTP, 0, "send failed");
 	closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	return(res);
     }
@@ -2036,6 +2100,7 @@ xmlNanoFTPGet(void *ctx, ftpDataCallback callback, void *userData,
 	    continue;
 	}
 	if ((len = recv(ctxt->dataFd, buf, sizeof(buf), 0)) < 0) {
+	    __xmlIOErr(XML_FROM_FTP, 0, "recv failed");
 	    callback(userData, buf, len);
 	    closesocket(ctxt->dataFd); ctxt->dataFd = -1;
 	    return(-1);
@@ -2068,12 +2133,13 @@ xmlNanoFTPRead(void *ctx, void *dest, int len) {
     if (len <= 0) return(0);
 
     len = recv(ctxt->dataFd, dest, len, 0);
+    if (len <= 0) {
+        __xmlIOErr(XML_FROM_FTP, 0, "recv failed");
+	xmlNanoFTPCloseConnection(ctxt);
+    }
 #ifdef DEBUG_FTP
     xmlGenericError(xmlGenericErrorContext, "Recvd %d bytes\n", len);
 #endif
-    if (len <= 0) {
-	xmlNanoFTPCloseConnection(ctxt);
-    }
     return(len);
 }
 
@@ -2155,10 +2221,10 @@ static
 void ftpData(void *userData, const char *data, int len) {
     if (userData == NULL) return;
     if (len <= 0) {
-	fclose(userData);
+	fclose((FILE*)userData);
 	return;
     }	
-    fwrite(data, len, 1, userData);
+    fwrite(data, len, 1, (FILE*)userData);
 }
 
 int main(int argc, char **argv) {
